# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-07-05 05:21
from __future__ import unicode_literals

from django.db import migrations

from dartcms.utils.loading import get_model


PageModule = get_model('pages', 'PageModule')

PAGE_MODULES = [
    {'slug': 'feeds', 'name': 'Feeds'},
    {'slug': 'contacts', 'name': 'Contacts'},
    {'slug': 'shop', 'name': 'Shop'},
    {'slug': 'page', 'name': 'Static Page'},
]


def insert_pagemodules(apps, schema):
    for m in PAGE_MODULES:
        PageModule.objects.create(**m)


def drop_pagemodules(apps, schema):
    for m in PAGE_MODULES:
        PageModule.objects.filter(slug=m['slug']).delete()


class Migration(migrations.Migration):
    dependencies = [
        ('pages', '0002_insert_homepage'),
    ]

    operations = [
        migrations.RunPython(insert_pagemodules, drop_pagemodules)
    ]



# -*- coding: utf-8 -*-
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext_lazy as _
from django.views.generic import ListView, CreateView, UpdateView, DeleteView, TemplateView
from django.contrib.messages.views import SuccessMessageMixin

from extra_views import CreateWithInlinesView, UpdateWithInlinesView

from .mixins import AdminMixin, JSONResponseMixin


class GridView(AdminMixin, ListView):
    template_name = 'dartcms/views/grid.html'
    paginate_by = None
    allow_empty = True
    grid_columns = (
        ('name', _('Title'), 'string', '70%'),
        ('date_created', _('Date created'), 'datetime', '30%'),
    )
    grid_actions = ()

    def get_queryset(self):
        if self.parent_kwarg_name:
            kwarg = self.kwargs[self.parent_kwarg_name]
            if self.parent_model_fk is not None:
                fk = self.parent_model_fk
            else:
                fk = '%s_id' % self.parent_model.__name__.lower()
            query_filter = '%s__exact' % fk

            queryset = self.model.objects.filter(**{
                query_filter: kwarg
            })
        else:
            queryset = super(GridView, self).get_queryset()

        return queryset

    def get_context_data(self, **kwargs):
        context = super(GridView, self).get_context_data(**kwargs)
        context['grid_columns'] = self.grid_columns
        context['grid_actions'] = self.grid_actions
        return context


class InsertObjectView(AdminMixin, SuccessMessageMixin, CreateView):
    template_name = 'dartcms/views/insert.html'
    success_message = _('Record successfully added')

    def form_valid(self, form):
        if self.parent_kwarg_name:
            obj = form.save(commit=False)

            if self.parent_model_fk is not None:
                fk = self.parent_model_fk
            else:
                fk = '%s_id' % self.parent_model.__name__.lower()

            setattr(obj, fk, self.kwargs[self.parent_kwarg_name])
            obj.save()
            form.save_m2m()
            self.object = obj
            return HttpResponseRedirect(self.get_success_url())
        else:
            return super(InsertObjectView, self).form_valid(form)

    def get_initial(self):
        initial = super(InsertObjectView, self).get_initial()
        if self.parent_model_fk and self.parent_kwarg_name in self.kwargs:
            initial.update({
                self.parent_model_fk: self.kwargs[self.parent_kwarg_name]
            })
        return initial


class InsertObjectWithInlinesView(AdminMixin, SuccessMessageMixin, CreateWithInlinesView):
    template_name = 'dartcms/views/insert.html'
    extra = 0
    success_message = _('Record successfully added')

    def forms_valid(self, form, inlines):
        if self.parent_kwarg_name:
            obj = form.save(commit=False)

            if self.parent_model_fk is not None:
                fk = self.parent_model_fk
            else:
                fk = '%s_id' % self.parent_model.__name__.lower()

            setattr(obj, fk, self.kwargs[self.parent_kwarg_name])
            obj.save()

            for formset in inlines:
                formset.save()

            return HttpResponseRedirect(self.get_success_url())
        else:
            return super(InsertObjectWithInlinesView, self).form_valid(form)

    def get_initial(self):
        initial = super(InsertObjectWithInlinesView, self).get_initial()
        if self.parent_model_fk and self.parent_kwarg_name in self.kwargs:
            initial.update({
                self.parent_model_fk: self.kwargs[self.parent_kwarg_name]
            })
        return initial


class UpdateObjectView(AdminMixin, SuccessMessageMixin, UpdateView):
    template_name = 'dartcms/views/update.html'
    success_message = _('Record successfully updated')


class UpdateObjectWithInlinesView(AdminMixin, SuccessMessageMixin, UpdateWithInlinesView):
    template_name = 'dartcms/views/update.html'
    extra = 0
    success_message = _('Record successfully updated')


class DeleteObjectView(AdminMixin, SuccessMessageMixin, DeleteView):
    template_name = 'dartcms/views/delete.html'
    success_message = _('Record successfully deleted')


class JSONView(JSONResponseMixin, TemplateView):
    def post(self, request, *args, **kwargs):
        return self.get(request, *args, **kwargs)

    def render_to_response(self, context, **response_kwargs):
        return self.render_to_json_response(context, **response_kwargs)


# coding: utf-8
import datetime

from django.utils.translation import ugettext_lazy as _
from django.db import models

from dartcms.utils.loading import is_model_registered


__all__ = [
    'FeedType',
    'Feed'
]


class FeedType(models.Model):
    def __unicode__(self):
        return self.name

    name = models.CharField(max_length=255, verbose_name=_('Name'))
    slug = models.SlugField(verbose_name=_('Slug'))


class Feed(models.Model):
    class Meta:
        ordering = ['name']

    def __unicode__(self):
        return self.name

    type = models.ForeignKey(FeedType, verbose_name=_('Type'), related_name='feeds')
    name = models.CharField(max_length=255, verbose_name=_('Name'))


class AbstractFeedItem(models.Model):
    class Meta:
        ordering = ['-date_published']
        abstract = True

    @property
    def search_content_type(self):
        return self.feed.type.slug

    def __unicode__(self):
        return self.name

    feed = models.ForeignKey(Feed, verbose_name=_('Feed'))
    name = models.CharField(max_length=1024, verbose_name=_('Title'))
    short_text = models.TextField(verbose_name=_('Short Text'))
    full_text = models.TextField(verbose_name=_('Full Text'))
    picture = models.ImageField(verbose_name=_('Picture'), upload_to='feeds/%Y/%m/%d')
    seo_keywords = models.TextField(
        verbose_name=_('Keywords'),
        help_text=_('Do not use more than 255 symbols'),
        blank=True
    )
    seo_description = models.TextField(
        verbose_name=_('Description'),
        help_text=_('Do not use more than 1024 symbols'),
        blank=True
    )
    is_visible = models.BooleanField(default=True, verbose_name=_('Show on Site'))
    date_published = models.DateTimeField(default=datetime.datetime.now, verbose_name=_('Date of Publication'))
    date_created = models.DateTimeField(auto_now_add=True, verbose_name=_('Date of Creation'))


if not is_model_registered('feeds', 'FeedItem'):
    class FeedItem(AbstractFeedItem):
        pass

    __all__.append('FeedItem')


# coding: utf-8
from django import forms
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _

from dartcms.apps.modules.models import Module


class UserForm(forms.ModelForm):
    class Meta:
        model = User
        exclude = ('password', 'last_login', 'date_joined', 'groups', 'user_permissions')

    modules = forms.ModelMultipleChoiceField(label=_(u"Modules"), queryset=Module.objects.all())

    def save(self, commit=True):
        obj = super(UserForm, self).save()
        data = self.cleaned_data

        if obj.id:
            obj.module_set.clear()

        for module in data.get('modules', []):
            obj.module_set.add(module)

        return obj



# coding: utf-8
import inspect
import importlib

from dartcms.utils.loading import get_model, is_model_registered

DARTCMS_CORE_APPS = [
    'django_gravatar',
    'mptt',

    'dartcms',
    'dartcms.apps.ads',
    'dartcms.apps.modules',
    'dartcms.apps.pages',
    'dartcms.apps.filemanager',
    'dartcms.apps.feeds'
]


def get_dartcms_core_apps(replacements=None):
    """
    If you want to override default DartCMS app, you should pass your app labels (as iterable) to this function.

    Usage:

    INSTALLED_APPS = [
        ...
    ] + get_dartcms_core_apps(['your_project.pages'])

    This will override default DartCMS app named 'pages'.
    """
    if not replacements:
        return DARTCMS_CORE_APPS

    def get_app_label(label, replacements):
        pattern = label.replace('dartcms.apps.', '')
        for replacement in replacements:
            if replacement.endswith(pattern):
                return replacement
        return label

    apps = []
    for app_label in DARTCMS_CORE_APPS:
        apps.append(get_app_label(app_label, replacements))

    return apps


def discover_models(app_label, existing_models):
    """
    Discovering models in certain app.
    If you want to override default DartCMS app, but you don't want to re-declare all models inside this app,
    you can call this function inside your custom app like this:

    Usage:

    # app: custom_app

    from django.db import models

    import dartcms

    from dartcms.apps.some_app.models import SomeAbstractModel


    __all__ = ['Model']

    __all__.extend(dartcms.discover_models('ads', __all__))


    class Model(SomeAbstractModel):
        custom_field = models.CharField(max_length=255)

    """
    from django.apps import apps
    from django.db import models

    caller = inspect.getmodule(inspect.stack()[1][0])
    module = importlib.import_module('dartcms.apps.%s.models' % app_label)

    m = []

    meta = type('Meta', (), {'app_label': app_label})

    for object_name in dir(module):
        klass = getattr(module, object_name)

        if inspect.isclass(klass) and issubclass(klass, models.Model):
            class_name = klass.__name__

            if class_name not in existing_models and class_name not in dir(caller):
                setattr(
                    caller,
                    class_name,
                    type(class_name, (klass, ), {
                        '__module__': app_label,
                        'Meta': meta
                    })
                )
                apps.register_model(app_label, getattr(caller, class_name))
                m.append(class_name)

    return m


# coding: utf-8
from django.conf import settings
from django.db import models
from django.db.models import F, Max
from django.db.models.signals import post_save, pre_delete, pre_save
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import get_language

from dartcms.utils.loading import get_model, is_model_registered
from mptt.models import MPTTModel, TreeForeignKey

__all__ = [
    'PageModule',
    'pre_save_handler',
    'post_save_handler',
    'pre_delete_handler'
]


class PageModule(models.Model):
    class Meta:
        ordering = ['id']
        verbose_name_plural = _('modules')
        verbose_name = _('module')

    def __unicode__(self):
        return self.name

    slug = models.SlugField(unique=True, verbose_name=_('Slug'))
    name = models.CharField(max_length=64, verbose_name=_('Name'))
    related_model = models.CharField(verbose_name=_('Related Model'), null=True, default=None, max_length=32)
    is_enabled = models.BooleanField(default=True, verbose_name=_('Is Enabled'))


class AbstractPage(MPTTModel):
    """
    Base page model. In common cases, you should not re-declare this class, since it have all necessary features.
    It holds base information about site's pages - urls, names, functional features, parents and childrens etc.
    """

    class Meta:
        abstract = True
        ordering = ['sort']
        verbose_name = _('page')
        verbose_name_plural = _('pages')
        unique_together = ['module', 'slug']

    def __unicode__(self):
        """
        Constructs breadcrumbs-like page name.
        """
        parent = self.parent
        page_names = [self.title]
        while parent:
            page_names.append(parent.title)
            parent = parent.parent
        return ' / '.join(reversed(page_names))

    parent = TreeForeignKey('self', null=True, related_name='children', verbose_name=_('Parent Page'), blank=True)

    title = models.CharField(max_length=255, verbose_name=_('Title'), help_text=_('Shows inside the title tag'))
    header = models.CharField(max_length=255, verbose_name=_('Page Header'))

    menu_name = models.CharField(max_length=255, default='', verbose_name=_('Menu name'))
    redirect_url = models.CharField(max_length=255, blank=True, default='', verbose_name=_('URL for Redirect'))

    slug = models.SlugField(default='', verbose_name=_('URL'), blank=True)
    url = models.CharField(max_length=512)

    sort = models.IntegerField(default=1)

    module = models.ForeignKey(PageModule, verbose_name=_('Module'), related_name='%(app_label)s_%(class)s_related')
    module_params = models.CharField(max_length=128, blank=True, null=True, default=None,
                                     verbose_name=_('Module parameters'))

    before_content = models.TextField(default='', blank=True, verbose_name=_('Before Content'))
    after_content = models.TextField(default='', blank=True, verbose_name=_('After Content'))

    seo_keywords = models.TextField(default='', blank=True, verbose_name=_('Keywords (meta keywords)'))
    seo_description = models.TextField(default='', blank=True, verbose_name=_('Description (meta description)'))

    ad_section = models.ForeignKey('ads.AdSection', verbose_name=_('Ads'), null=True, blank=True,
                                   related_name='%(app_label)s_%(class)s_related')

    is_enabled = models.BooleanField(default=True, verbose_name=_('Is Enabled'))
    is_in_menu = models.BooleanField(default=True, verbose_name=_('Show in Menu'))

    date_created = models.DateTimeField(auto_now_add=True)

    @property
    def menu_url(self):
        lang = get_language()
        prefix = '' if lang == settings.LANGUAGE_CODE else '/%s' % lang
        return prefix + self.menu_url if self.menu_url else prefix + self.url

    @property
    def page_url(self):
        if self.module.slug == 'homepage':
            return '/'
        return '/%s/%s/' % (self.module.slug, self.slug)

    def serializable_object(self):
        obj = {
            'pk': self.pk,
            'parent_id': self.parent_id,
            'title': self.title,
            'module': str(self.module),
            'url': self.url,
            'children': []
        }
        for child in self.get_children():
            obj['children'].append(child.serializable_object())
        return obj


def pre_save_handler(sender, **kwargs):
    """
    Sorting pages while saving.
    """
    instance = kwargs.get('instance')
    if instance:
        instance.url = instance.page_url

        current_level_pages = sender.objects.filter(parent=instance.parent)

        if not current_level_pages.exists():
            instance.sort = 1

        if not instance.id or instance.sort == 0:
            max_sort = current_level_pages.aggregate(Max('sort'))
            s = max_sort.get('sort_max')
            instance.sort = s + 1 if s else 1
        else:
            page_cache = sender.objects.get(pk=instance.id)

            if page_cache.parent == instance.parent:
                if int(page_cache.sort) < int(instance.sort):
                    instance.sort -= 1
                    current_level_pages.filter(sort__gt=page_cache.sort, sort__lte=instance.sort).update(
                        sort=F('sort') - 1)
                else:
                    current_level_pages.filter(sort__gte=instance.sort, sort__lt=page_cache.sort).update(
                        sort=F('sort') + 1)
            else:
                current_level_pages.filter(sort__gte=instance.sort).update(sort=F('sort') + 1)
                sender.objects.filter(parent=page_cache.parent, sort__gt=page_cache.sort).update(sort=F('sort') - 1)


def post_save_handler(**kwargs):
    instance = kwargs.get('instance')
    if instance:
        children = instance.children.all()
        if children:
            for child in instance.children.all():
                child.save()


def pre_delete_handler(sender, **kwargs):
    instance = kwargs.get('instance')
    if instance:
        sender.objects.filter(parent=instance.parent, sort__gt=instance.sort).update(sort=F('sort') - 1)


if is_model_registered('pages', 'Page'):
    page_model = get_model('pages', 'Page')
else:
    class Page(AbstractPage):
        pass

    __all__.append('Page')

    page_model = Page

pre_save.connect(pre_save_handler, sender=page_model)
post_save.connect(post_save_handler, sender=page_model)
pre_delete.connect(pre_delete_handler, sender=page_model)



# coding: utf-8
import re

from django.core.urlresolvers import reverse
from django.http import Http404, JsonResponse


class ModulePermissionsMixin(object):
    def check_module_perms(self):
        root = reverse('dartcms:dashboard:index')
        active_module_slug = self.request.path.replace(root, '').strip('/').split('/')[0]
        user_modules = self.request.user.module_set.all().values_list('slug', flat=True)
        return active_module_slug in user_modules

    def dispatch(self, request, *args, **kwargs):
        if not self.check_module_perms():
            raise Http404

        response = super(ModulePermissionsMixin, self).dispatch(request, *args, **kwargs)
        return response


class AdminMixin(ModulePermissionsMixin):
    # Parent model for children element, used in insert view to create foreign key relation.
    parent_model = None
    parent_model_fk = None

    # URL argument name to get the parent object in children.
    parent_kwarg_name = ''

    # Page header. If page_header == '', model's verbose name used.
    page_header = ''

    # Index URL
    index_url = ''
    success_url = ''

    def get_context_data(self, *args, **kwargs):
        context = super(AdminMixin, self).get_context_data(*args, **kwargs)

        self.index_url = re.sub(r'(insert/\d+/|insert/|update/\d+/|delete/(\d+)/)', '', self.request.path)
        if self.parent_kwarg_name:
            reg = r'(%s/(\d+)/)$' % self.kwargs['children_url']
            parent_url = re.sub(reg, '', self.request.path)
        else:
            parent_url = ''

        context.update({
            'page_header': self.page_header if self.page_header else self.model._meta.verbose_name_plural,
            'parent_kwarg_name': self.parent_kwarg_name,
            'index_url': self.index_url,
            'parent_url': parent_url
        })
        return context

    def get_success_url(self):
        if self.success_url:
            return self.success_url

        self.index_url = re.sub(r'(insert/\d+/|insert/|update/\d+/|delete/(\d+)/)', '', self.request.path)
        return self.index_url


class JSONResponseMixin(object):
    """
    A mixin that can be used to render a JSON response.
    """
    http_method_names = ['get', 'post']

    def render_to_json_response(self, context, **response_kwargs):
        """
        Returns a JSON response, transforming 'context' to make the payload.
        """
        safe = response_kwargs.pop('safe', None) or False
        return JsonResponse(self.get_data(context), safe=safe, **response_kwargs)

    def get_data(self, context):
        """
        Returns an object that will be serialized as JSON by json.dumps().
        """
        return context


# coding: utf-8
from django import forms
from django.utils.translation import ugettext_lazy as _

from dartcms.utils.loading import get_model
from form_utils.forms import BetterModelForm

PageModule = get_model('pages', 'PageModule')
AdSection = get_model('ads', 'AdSection')


class PageForm(BetterModelForm):
    class Meta:
        model = get_model('pages', 'Page')
        exclude = ['url', 'date_created', 'date_changed', 'sort']
        widgets = {
            'before_content': forms.Textarea(attrs={'rows': 20, 'class': 'rte'}),
            'after_content': forms.Textarea(attrs={'rows': 20, 'class': 'rte'}),
            'module_params': forms.Select()
        }

        fieldsets = (
            ('main', {'fields': (
                'parent',
                'slug',
                'title',
                'header',
                'menu_name',
                'menu_url',
                'module',
                'module_params',
            ), 'legend': _('Main')}),
            ('content', {'fields': (
                'before_content',
                'after_content',
            ), 'legend': _('Content')}),
            ('ads', {'fields': (
                'ad_section',
            ), 'legend': _('Ads')}),
            ('seo', {'fields': (
                'seo_keywords',
                'seo_description',
            ), 'legend': _('SEO')}),
            ('advanced', {'fields': (
                'is_enabled',
                'is_in_menu',
                'is_locked',
            ), 'legend': _('Advanced')}))


    module = forms.ModelChoiceField(PageModule.objects.filter(is_enabled=True), empty_label=None)
    ad_section = forms.ModelChoiceField(AdSection.objects.filter(is_enabled=True), required=False, label=_('Ads'))

    def __init__(self, *args, **kwargs):
        super(PageForm, self).__init__(*args, **kwargs)
        if self.instance.pk and self.instance.module.slug == 'homepage':
            del self.fields['is_enabled']
            del self.fields['redirect_url']
            del self.fields['module']
            del self.fields['parent']
            del self.fields['module_params']
            del self.fields['slug']



#!/usr/bin/env python2.7
# pylint: disable=C0301
from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

from contextlib import contextmanager
from datetime import datetime
from hashlib import sha256
from json import dump as save
from json import load
from os import remove as _delete_file
from os import chdir, environ, stat
from os.path import abspath, basename, dirname, expandvars, realpath
from subprocess import check_call as run
from sys import argv

from boto.exception import S3ResponseError
from boto.s3.connection import S3Connection
from boto.s3.key import Key

CONFIG_FILE = './S3Cachefile.json'
UPLOAD_TODO_FILE = './S3CacheTodo.json'
BYTES_PER_MB = 1024 * 1024


@contextmanager
def timer():
    start = datetime.utcnow()
    yield
    end = datetime.utcnow()
    elapsed = end - start
    print("\tDone. Took", int(elapsed.total_seconds()), "second(s).")


@contextmanager
def todo_file(writeback=True):
    try:
        with open(UPLOAD_TODO_FILE, 'rt') as json_file:
            todo = load(json_file)
    except (IOError, OSError, ValueError):
        todo = {}

    yield todo

    if writeback:
        try:
            with open(UPLOAD_TODO_FILE, 'wt') as json_file:
                save(todo, json_file)
        except (OSError, IOError) as save_err:
            print("Error saving {}:".format(UPLOAD_TODO_FILE), save_err)


def _sha256_of_file(filename):
    hasher = sha256()
    with open(filename, 'rb') as input_file:
        hasher.update(input_file.read())
    file_hash = hasher.hexdigest()
    print('sha256({}) = {}'.format(filename, file_hash))
    return file_hash


def _delete_file_quietly(filename):
    try:
        _delete_file(filename)
    except (OSError, IOError):
        pass


def mark_needs_uploading(cache_name):
    with todo_file() as todo:
        todo[cache_name] = True


def mark_uploaded(cache_name):
    with todo_file() as todo:
        todo.pop(cache_name, None)


def need_to_upload(cache_name):
    with todo_file(writeback=False) as todo:
        return todo.get(cache_name, False)


def _tarball_size(directory):
    kib = stat(_tarball_filename_for(directory)).st_size // BYTES_PER_MB
    return "{} MiB".format(kib)


def _tarball_filename_for(directory):
    return abspath('./{}.tar.gz'.format(basename(directory)))


def _create_tarball(directory):
    print("Creating tarball of {}...".format(directory))
    with timer():
        run(['tar', '-czf', _tarball_filename_for(directory), '-C', dirname(directory), basename(directory)])


def _extract_tarball(directory):
    print("Extracting tarball of {}...".format(directory))
    with timer():
        run(['tar', '-xzf', _tarball_filename_for(directory), '-C', dirname(directory)])


def download(directory):
    mark_uploaded(cache_name)  # reset
    try:
        print("Downloading {} tarball from S3...".format(cache_name))
        with timer():
            key.get_contents_to_filename(_tarball_filename_for(directory))
    except S3ResponseError as err:
        mark_needs_uploading(cache_name)
        raise SystemExit("Cached {} download failed!".format(cache_name))
    print("Downloaded {}.".format(_tarball_size(directory)))
    _extract_tarball(directory)
    print("{} successfully installed from cache.".format(cache_name))


def upload(directory):
    _create_tarball(directory)
    print("Uploading {} tarball to S3... ({})".format(cache_name, _tarball_size(directory)))
    with timer():
        key.set_contents_from_filename(_tarball_filename_for(directory))
    print("{} cache successfully updated.".format(cache_name))
    mark_uploaded(cache_name)


if __name__ == '__main__':
    # Uses environment variables:
    #   AWS_ACCESS_KEY_ID -- AWS Access Key ID
    #   AWS_SECRET_ACCESS_KEY -- AWS Secret Access Key
    argv.pop(0)
    if len(argv) != 2:
        raise SystemExit("USAGE: s3_cache.py <download | upload> <cache name>")
    mode, cache_name = argv
    script_dir = dirname(realpath(__file__))
    chdir(script_dir)
    try:
        with open(CONFIG_FILE, 'rt') as config_file:
            config = load(config_file)
    except (IOError, OSError, ValueError) as config_err:
        print(config_err)
        raise SystemExit("Error when trying to load config from JSON file!")

    try:
        cache_info = config[cache_name]
        key_file = expandvars(cache_info["key"])
        fallback_cmd = cache_info["generate"]
        directory = expandvars(cache_info["cache"])
    except (TypeError, KeyError) as load_err:
        print(load_err)
        raise SystemExit("Config for cache named {!r} is missing or malformed!".format(cache_name))

    try:
        try:
            BUCKET_NAME = environ['TWBS_S3_BUCKET']
        except KeyError:
            raise SystemExit("TWBS_S3_BUCKET environment variable not set!")

        conn = S3Connection()
        bucket = conn.lookup(BUCKET_NAME)
        if bucket is None:
            raise SystemExit("Could not access bucket!")

        key_file_hash = _sha256_of_file(key_file)

        key = Key(bucket, key_file_hash)
        key.storage_class = 'REDUCED_REDUNDANCY'

        if mode == 'download':
            download(directory)
        elif mode == 'upload':
            if need_to_upload(cache_name):
                upload(directory)
            else:
                print("No need to upload anything.")
        else:
            raise SystemExit("Unrecognized mode {!r}".format(mode))
    except BaseException as exc:
        if mode != 'download':
            raise
        print("Error!:", exc)
        print("Unable to download from cache.")
        print("Running fallback command to generate cache directory {!r}: {}".format(directory, fallback_cmd))
        with timer():
            run(fallback_cmd, shell=True)



# coding: utf-8
class DartCMSConfig(object):
    config = {}

    def __init__(self, config):
        self.config = config

    @property
    def base(self):
        return {
            'parent_kwarg_name': self.config.get('parent_kwarg_name'),
            'parent_model_fk': self.config.get('parent_model_fk'),
            'parent_model': self.config.get('parent_model'),
            'model': self.config['model'],
        }

    @property
    def grid(self):
        config = self.base
        config.update(self.config['grid'])
        return config

    @property
    def form(self):
        config = self.base
        config.update(self.config['form'])
        return config



# coding: utf-8
from django.conf import settings

from django.conf.urls import include, url
from django.views.i18n import javascript_catalog

js_info_dict = {
    'packages': ('dartcms',),
}

urlpatterns = [
    url(r'^i18n/', include('django.conf.urls.i18n')),
    url(r'^jsi18n/$', javascript_catalog, js_info_dict, name='javascript-catalog'),

    url(r'^', include('dartcms.apps.dashboard.urls', namespace='dashboard')),
    url(r'^auth/', include('dartcms.apps.auth.urls', namespace='auth')),
    url(r'^cms-users/', include('dartcms.apps.users.urls', namespace='users')),
    url(r'^sitemap/', include('dartcms.apps.pages.urls', namespace='pages')),
    url(r'^feeds/', include('dartcms.apps.feeds.urls', namespace='feeds')),
    url(r'^filemanager/', include('dartcms.apps.filemanager.urls', namespace='filemanager')),
]

additional_apps = getattr(settings, 'DARTCMS_ADDITIONAL_APPS_URLPATTERNS', [])

if additional_apps:
    additional_patterns = []
    for app in additional_apps:
        additional_patterns.append(url(r'^%s/' % app[0], include(app[1], namespace=app[2])))

    urlpatterns += additional_patterns



# -*- coding: utf-8 -*-
from django.http import Http404

from dartcms.utils.loading import get_model

from dartcms.views import GridView, InsertObjectView, JSONView
from dartcms.views.mixins import JSONResponseMixin, ModulePermissionsMixin


class GetTreeView(GridView, JSONResponseMixin):
    def post(self, request, *args, **kwargs):
        return self.get(request, *args, **kwargs)

    def render_to_response(self, context, **response_kwargs):
        return self.render_to_json_response(context, safe=False, **response_kwargs)

    def get_data(self, context):
        homepage = self.object_list.get(module__slug='homepage')
        tree = [homepage.serializable_object()]
        return tree


class InsertPageView(InsertObjectView):
    def get_initial(self):
        return {'parent': self.model.objects.filter(pk=self.request.GET.get('parent', 0)).first()}


class TreeActionView(ModulePermissionsMixin, JSONView):
    def get_pages(self):
        target_id = self.request.GET.get('target')
        source_id = self.request.GET.get('source')

        if target_id and source_id:
            Page = get_model('pages', 'Page')

            target = Page.objects.get(pk=target_id)
            source = Page.objects.get(pk=source_id)

            return target, source

        raise Http404


class AppendPageView(TreeActionView):
    def get_data(self, context):
        target, source = self.get_pages()
        source.move_to(target, position='last-child')
        return {'result': True}


class MovePageView(TreeActionView):
    def get_data(self, context):
        position = self.request.GET.get('position')
        target, source = self.get_pages()
        source.move_to(target, position=position)
        return {'result': True}


class LoadModuleParamsView(ModulePermissionsMixin, JSONView):
    def get_data(self, context):
        PageModule = get_model('pages', 'PageModule')
        Page = get_model('pages', 'Page')

        module = PageModule.objects.get(pk=self.request.GET.get('selected_module', 0))

        if module.related_model:
            model_path = module.related_model.split('.')
            app = model_path[0]
            model = model_path[1]
            try:
                RelatedModel = get_model(app, model)
                data = RelatedModel.objects.all()
            except (LookupError, ImportError):
                data = []
        else:
            data = []

        page_pk = self.request.GET.get('pk', 0)

        if page_pk != '':
            page = Page.objects.get(pk=page_pk)
        else:
            page = None

        params = []

        for item in data:
            if page is not None and page.module == module and str(page.module_params) == str(item.pk):
                params.append({'label': str(item), 'value': item.pk, 'selected': True})
            else:
                params.append({'label': str(item), 'value': item.pk})

        return {
            'result': True,
            'data': params
        }



# -*- coding: utf-8 -*-
from django.conf.urls import url
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _

from dartcms.utils.config import DartCMSConfig
from dartcms.views import DeleteObjectView, GridView
from forms import UserForm
from views import ChangePasswordView, CMSUserInsertView, CMSUserUpdateView

config = DartCMSConfig({
    'model': User,
    'grid': {
        'grid_columns': (
            ('username', _('Username'), 'string', '60%'),
            ('last_login', _('Last Login'), 'datetime', '20%'),
            ('is_staff', _('Staff'), 'boolean', '10%'),
            ('is_active', _('Active'), 'boolean', '10%'),
        ),
        'grid_actions': (
            ('change-password', _('Change Password'), 'edit'),
        )
    },
    'form': {
        'form_class': UserForm
    }
})

urlpatterns = [
    url(r'^$', GridView.as_view(**config.grid), name='index'),
    url(r'^insert/$', CMSUserInsertView.as_view(**config.form), name='insert'),
    url(r'^update/(?P<pk>\d+)/$', CMSUserUpdateView.as_view(**config.form), name='update'),
    url(r'^delete/(?P<pk>\d+)/$', DeleteObjectView.as_view(**config.base), name='delete'),
    url(r'^change-password/(?P<pk>\d+)/$', ChangePasswordView.as_view(), name='change_password'),
]



# coding: utf-8
import datetime

from django import forms
from django.utils.translation import ugettext_lazy as _

from dartcms.utils.loading import get_model


class FeedItemForm(forms.ModelForm):
    class Meta:
        model = get_model('feeds', 'FeedItem')
        exclude = ['feed', 'slug']
        widgets = {
            'short_text': forms.Textarea(attrs={'class': 'rte'}),
            'full_text': forms.Textarea(attrs={'class': 'rte'}),
        }

    date_published = forms.DateTimeField(
        widget=forms.DateTimeInput(
            attrs={
                'class': 'datetime'
            }
        ),
        localize=True,
        initial=datetime.datetime.now,
        label=_('Date of publication')
    )


# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-05-20 02:26
from __future__ import unicode_literals

from django.db import migrations

from dartcms.apps.modules.models import Module, ModuleGroup

MODULE_GROUPS = [
    {
        'sort': 1,
        'description': '',
        'fa': 'fa-flag',
        'slug': 'admin',
        'name': 'Administration',
        'modules': [
            {
                'sort': 1,
                'is_enabled': True,
                'name': 'CMS Users',
                'slug': 'cms-users',
                'description': ''
            }
        ]
    },
    {
        'sort': 2,
        'description': '',
        'fa': 'fa-file',
        'slug': 'content',
        'name': 'Content',
        'modules': [
            {
                'sort': 1,
                'is_enabled': True,
                'name': 'Site Structure',
                'slug': 'sitemap',
                'description': ''
            },
            {
                'sort': 2,
                'is_enabled': True,
                'name': 'Feeds',
                'slug': 'feeds',
                'description': ''
            }
        ]
    },
    {
        'sort': 3,
        'description': '',
        'fa': 'fa-cog',
        'slug': 'service',
        'name': 'Services',
        'modules': [
            {
                'sort': 1,
                'is_enabled': True,
                'name': 'Contacts',
                'slug': 'contacts',
                'description': ''
            }
        ]
    }
]


def insert_modules(apps, schema):
    for group in MODULE_GROUPS:
        group_modules = group.pop('modules', [])
        group = ModuleGroup.objects.create(**group)
        for module in group_modules:
            module['group'] = group
            Module.objects.create(**module)


def delete_modules(apps, schema):
    for group in MODULE_GROUPS:
        ModuleGroup.objects.get(slug=group['slug']).delete()


class Migration(migrations.Migration):
    dependencies = [
        ('modules', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(insert_modules, delete_modules)
    ]



# -*- coding: utf-8 -*-
from django.conf.urls import url, include
from django.forms import modelform_factory
from django.utils.translation import ugettext_lazy as _

from dartcms.utils.config import DartCMSConfig
from dartcms.utils.loading import get_model
from dartcms.views import GridView, UpdateObjectView, DeleteObjectView, InsertObjectView

Feed = get_model('feeds', 'Feed')

config = DartCMSConfig({
    'model': Feed,
    'grid': {
        'grid_columns': (
            ('type', _('Type'), 'string', '30%'),
            ('name', _('Name'), 'string', '70%'),
        ),
        'grid_actions': (
            ('items', _('View Records'), 'next'),
        ),
    },
    'form': {
        'form_class': modelform_factory(Feed, exclude=[]),
    }
})

urlpatterns = [
    url(r'^$', GridView.as_view(**config.grid), name='index'),
    url(r'^insert/$', InsertObjectView.as_view(**config.form), name='insert'),
    url(r'^update/(?P<pk>\d+)/$', UpdateObjectView.as_view(**config.form), name='update'),
    url(r'^delete/(?P<pk>\d+)/$', DeleteObjectView.as_view(**config.base), name='delete'),
    url(r'^(?P<children_url>items)/(?P<feed>\d+)/', include('dartcms.apps.feeds.items.urls', namespace='feed_items')),
]



# coding: utf-8
def modules_data(request):
    if not request.is_ajax() and request.user.is_authenticated() and request.user.is_staff:
        modules = request.user.module_set.all()

        module_groups = [m.group for m in modules]
        module_groups_set = set(module_groups)

        return {
            'module_groups': module_groups_set
        }
    else:
        return {}



# -*- coding: utf-8 -*-
from django.conf.urls import url, include
from django.utils.translation import ugettext_lazy as _

from dartcms.utils.config import DartCMSConfig
from dartcms.utils.loading import get_model
from dartcms.views import GridView, UpdateObjectView, DeleteObjectView, InsertObjectView

from .forms import FeedItemForm

FeedItem = get_model('feeds', 'FeedItem')

config = DartCMSConfig({
    'model': FeedItem,
    'parent_kwarg_name': 'feed',
    'parent_model_fk': 'feed_id',
    'grid': {
        'grid_columns': (
            ('name', _('Name'), 'string', '60%'),
            ('is_visible', _('Is Visible'), 'boolean', '20%'),
            ('date_published', _('Date of Publication'), 'datetime', '20%'),
        )
    },
    'form': {
        'form_class': FeedItemForm,
    }
})

urlpatterns = [
    url(r'^$', GridView.as_view(**config.grid), name='index'),
    url(r'^insert/$', InsertObjectView.as_view(**config.form), name='insert'),
    url(r'^update/(?P<pk>\d+)/$', UpdateObjectView.as_view(**config.form), name='update'),
    url(r'^delete/(?P<pk>\d+)/$', DeleteObjectView.as_view(**config.base), name='delete'),
]
